local major, minor = "Graph-1.0", "$Revision: 1 $"

if not AceLibrary then error(major .. " requires AceLibrary.") end
if not AceLibrary:IsNewVersion(major, minor) then return end

local lib={}

function lib:SetYMax(ymax)
	self.YMax=ymax

	if self.RefreshGraph then
		self:RefreshGraph()
	end
end

function lib:SetYAxis(ymin,ymax)
	self.YMin=ymin
	self.YMax=ymax

	if self.RefreshGraph then
		self:RefreshGraph()
	end
end

function lib:SetXAxis(xmin,xmax)
	self.XMin=xmin
	self.XMax=xmax

	if self.RefreshGraph then
		self:RefreshGraph()
	end
end

function lib:AddTimeData(value)
	local t={}
	t.Time=GetTime()
	t.Value=value
	table.insert(self.Data,t)
end

--Realtime Graph
--Performs a convolution in realtime allowing to graph Framerate, DPS, or any other data you want graphed in realtime

function lib:OnUpdateGraphRealtime()
	local i,j
	local CurTime=GetTime()
	local MaxBarHeight=self:GetHeight()
	
	--Slow Mode performs an entire convolution every frame
	if self.Mode=="SLOW" then
		--Initialize Bar Data
		self.BarHeight={}
		for i=1,self.BarNum do
			self.BarHeight[i]=0
		end
		local k,v
		local BarTimeRadius=(self.XMax-self.XMin)/self.BarNum
		local DataValue=1/(2*self.TimeRadius)

		--Take the convolution of the dataset on to the bars wtih a rectangular filter
		for k,v in pairs(self.Data) do
			if v.Time<(CurTime+self.XMin-self.TimeRadius) then
				table.remove(self.Data,k)	
			else	
				local DataTime=v.Time-CurTime
				local LowestBar=math.max(math.floor((DataTime-self.XMin-self.TimeRadius)/BarTimeRadius),1)
				local HighestBar=math.min(math.ceil((DataTime-self.XMin+self.TimeRadius)/BarTimeRadius),self.BarNum)
				for i=LowestBar,HighestBar do
					self.BarHeight[i]=self.BarHeight[i]+v.Value*DataValue
				end
			end
		end
	end
	for i,bar in pairs(self.Bars) do
		local h
		h=math.max(math.min(MaxBarHeight*(self.BarHeight[i]-self.YMin)/(self.YMax-self.YMin),MaxBarHeight),0)
		
		if h~=0 then
			bar:SetHeight(h)
			bar:Show()
		else
			bar:Hide()
		end
	end
end

function lib:CreateGraphRealtime(name,parent,relative,relativeTo,offsetX,offsetY,Width,Height)
	local graph
	local i
	graph = CreateFrame("Frame",name,parent)

	
	graph:SetPoint(relative,parent,relativeTo,offsetX,offsetY)
	graph:SetWidth(Width)
	graph:SetHeight(Height)
	graph:Show()

	--Create the bars
	graph.Bars={}
	graph.BarNum=Width
	for i=1,Width do
		local bar
		bar = graph:CreateTexture(nil,"BACKGROUND")
		bar:SetPoint("BOTTOMLEFT",graph,"BOTTOMLEFT",i-1,0)
		bar:SetHeight(1)
		bar:SetWidth(1)
		bar:SetGradientAlpha("VERTICAL",0.2,0.0,0.0,0.5,1.0,0.0,0.0,1.0)
		bar:Show()
		bar:SetTexture("Interface\\Buttons\\WHITE8X8.blp")
		table.insert(graph.Bars,bar)
	end

	
	
	--Set the various functions
	graph.SetXAxis=self.SetXAxis
	graph.SetYMax=self.SetYMax
	graph.AddTimeData=self.AddTimeData
	graph.OnUpdate=self.OnUpdateGraphRealtime

	--Set the update function
	graph:SetScript("OnUpdate", graph.OnUpdate)

	--Initialize Data
	graph.YMax=60
	graph.YMin=0
	graph.XMax=-0.75
	graph.XMin=-10
	graph.TimeRadius=0.5
	graph.Mode="SLOW"
	graph.Data={}
	
	return graph
end


--Line Graph
function lib:CreateGraphLine(name,parent,relative,relativeTo,offsetX,offsetY,Width,Height)
	local graph
	local i
	graph = CreateFrame("Frame",name,parent)

	
	graph:SetPoint(relative,parent,relativeTo,offsetX,offsetY)
	graph:SetWidth(Width)
	graph:SetHeight(Height)
	graph:Show()


	
	
	--Set the various functions
	graph.SetXAxis=self.SetXAxis
	graph.SetYAxis=self.SetYAxis
	graph.AddDataSeries=self.AddDataSeries
	graph.ResetData=self.ResetData
	graph.RefreshGraph=self.RefreshLineGraph

	graph.DrawLine=self.DrawLine
	graph.HideLines=self.HideLines


	--Initialize Data
	graph.YMax=1
	graph.YMin=0
	graph.XMax=1
	graph.XMin=0
	graph.Data={}
	
	return graph
end

function lib:AddDataSeries(points,color)
	table.insert(self.Data,{Points=points;Color=color})
	
	if self.RefreshGraph then
		self:RefreshGraph()
	end
end

function lib:ResetData()
	self.Data={}
	
	if self.RefreshGraph then
		self:RefreshGraph()
	end
end

function lib:RefreshLineGraph()
	local k1, k2, series
	self:HideLines(self)

	local Width=self:GetWidth()
	local Height=self:GetWidth()

	for k1, series in pairs(self.Data) do
		local LastPoint
		LastPoint=nil
		--DEFAULT_CHAT_FRAME:AddMessage("Data Series "..k1)
		for k2, point in pairs(series.Points) do
			if LastPoint then
				local TPoint={x=point[1];y=point[2]}

				TPoint.x=Width*(TPoint.x-self.XMin)/(self.XMax-self.XMin)
				TPoint.y=Width*(TPoint.y-self.YMin)/(self.YMax-self.YMin)

				self:DrawLine(self,LastPoint.x,LastPoint.y,TPoint.x,TPoint.y,16,series.Color)

				LastPoint=TPoint
			else
				LastPoint={x=point[1];y=point[2]}
				LastPoint.x=Width*(LastPoint.x-self.XMin)/(self.XMax-self.XMin)
				LastPoint.y=Width*(LastPoint.y-self.YMin)/(self.YMax-self.YMin)
			end
			--DEFAULT_CHAT_FRAME:AddMessage("Point "..k2.." "..LastPoint.x..", "..LastPoint.y)
		end
	end
end

--Copied from Blizzard's TaxiFrame code and modifed for IMBA then remodified for GraphLib

-- The following function is used with permission from Daniel Stephens <iriel@vigilance-committee.org>
TAXIROUTE_LINEFACTOR = 128/126; -- Multiplying factor for texture coordinates
TAXIROUTE_LINEFACTOR_2 = TAXIROUTE_LINEFACTOR / 2; -- Half o that

-- T        - Texture
-- C        - Canvas Frame (for anchoring)
-- sx,sy    - Coordinate of start of line
-- ex,ey    - Coordinate of end of line
-- w        - Width of line
-- relPoint - Relative point on canvas to interpret coords (Default BOTTOMLEFT)
--/script IMBA_DrawLine(IMBA_Ouro,0,0,-200,-200,32)
function lib:DrawLine(C, sx, sy, ex, ey, w, color)
	local T, lineNum, relPoint;
	if (not relPoint) then relPoint = "BOTTOMLEFT"; end


	if not C.GraphLib_Lines then
		C.GraphLib_Lines={};
		C.GraphLib_LineData={};
	end

	T=nil;

	for k,v in pairs(C.GraphLib_Lines) do
		if not v:IsShown() and not T then
			T=v;
			lineNum=k;
			T:Show();		
		end
	end

	if not T then
		T=C:CreateTexture(C:GetName().."_Line"..(getn(C.GraphLib_Lines)+1), "ARTWORK");
		T:SetTexture("Interface\\AddOns\\GraphLib\\textures\\line");
		tinsert(C.GraphLib_Lines,T);
		tinsert(C.GraphLib_LineData,{sx,sy,ex,ey,w,{color[1],color[2],color[3],color[4]}});
	else
		C.GraphLib_LineData[lineNum]={sx,sy,ex,ey,w,{color[1],color[2],color[3],color[4]}};
	end

	T:SetVertexColor(color[1],color[2],color[3],color[4]);
	-- Determine dimensions and center point of line
	local dx,dy = ex - sx, ey - sy;
	local cx,cy = (sx + ex) / 2, (sy + ey) / 2;

	-- Normalize direction if necessary
	if (dx < 0) then
		dx,dy = -dx,-dy;
	end

	-- Calculate actual length of line
	local l = sqrt((dx * dx) + (dy * dy));

	-- Quick escape if it's zero length
	if (l == 0) then
		T:SetTexCoord(0,0,0,0,0,0,0,0);
		T:SetPoint("BOTTOMLEFT", C, relPoint, cx,cy);
		T:SetPoint("TOPRIGHT",	C, relPoint, cx,cy);
		return;
	end

	-- Sin and Cosine of rotation, and combination (for later)
	local s,c = -dy / l, dx / l;
	local sc = s * c;

	-- Calculate bounding box size and texture coordinates
	local Bwid, Bhgt, BLx, BLy, TLx, TLy, TRx, TRy, BRx, BRy;
	if (dy >= 0) then
		Bwid = ((l * c) - (w * s)) * TAXIROUTE_LINEFACTOR_2;
		Bhgt = ((w * c) - (l * s)) * TAXIROUTE_LINEFACTOR_2;
		BLx, BLy, BRy = (w / l) * sc, s * s, (l / w) * sc;
		BRx, TLx, TLy, TRx = 1 - BLy, BLy, 1 - BRy, 1 - BLx; 
		TRy = BRx;
	else
		Bwid = ((l * c) + (w * s)) * TAXIROUTE_LINEFACTOR_2;
		Bhgt = ((w * c) + (l * s)) * TAXIROUTE_LINEFACTOR_2;
		BLx, BLy, BRx = s * s, -(l / w) * sc, 1 + (w / l) * sc;
		BRy, TLx, TLy, TRy = BLx, 1 - BRx, 1 - BLx, 1 - BLy;
		TRx = TLy;
	end

	-- Set texture coordinates and anchors
	T:ClearAllPoints();
	T:SetTexCoord(TLx, TLy, BLx, BLy, TRx, TRy, BRx, BRy);
	T:SetPoint("BOTTOMLEFT", C, relPoint, cx - Bwid, cy - Bhgt);
	T:SetPoint("TOPRIGHT",	C, relPoint, cx + Bwid, cy + Bhgt);

	return T:GetName();
end

function lib:HideLines(C)
	if not C.GraphLib_Lines then
		return
	end

	for k,v in pairs(C.GraphLib_Lines) do
		v:Hide()
	end
end














AceLibrary:Register(lib, major, minor)
















--Test Functions
function TestGraphLib()
	local Graph=AceLibrary("Graph-1.0")
	g=Graph:CreateGraphRealtime("testgraph",UIParent,"LEFT","LEFT",0,0,150,150)

	f = CreateFrame("Frame",name,parent)
	f:SetScript("OnUpdate",function() g:AddTimeData(1) end)
	f:Show()
end


function TestGraphLib2()
	local Graph=AceLibrary("Graph-1.0")
	g=Graph:CreateGraphRealtime("testgraph",UIParent,"LEFT","LEFT",0,0,150,150)
	g.YMax=10
	g.XMin=-50


	f = CreateFrame("Frame",name,parent)
	f:SetScript("OnEvent",function() g:AddTimeData(1) end)
	f:RegisterEvent("CHAT_MSG_ADDON")
	f:Show()
end

function TestGraphLib3()
	local Graph=AceLibrary("Graph-1.0")
	g=Graph:CreateGraphLine("testgraph",UIParent,"LEFT","LEFT",0,0,150,150)


	g:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                            tile = true, tileSize = 16, edgeSize = 16, 
                                            insets = { left = 4, right = 4, top = 4, bottom = 4 }});
	g:SetBackdropColor(0,0,0,0.5);

	local Data1={{0.05,0.05},{0.2,0.3},{0.4,0.2},{0.9,0.6}}
	local Data2={{0.05,0.8},{0.3,0.1},{0.5,0.4},{0.95,0.05}}

	g:AddDataSeries(Data1,{1.0,0.0,0.0,0.8})
	g:AddDataSeries(Data2,{0.0,1.0,0.0,0.8})

end